@@ -14,15 +14,16 @@ String::Copyright - Representation of text-based copyright statements
 
 =head1 VERSION
 
-Version 0.003001
+Version 0.003002
 
 =cut
 
-our $VERSION = '0.003001';
+our $VERSION = '0.003002';
 
 # Dependencies
 use parent 'Exporter::Tiny';
 use Carp ();
+use Number::Range;
 
 our @EXPORT = qw/copyright/;
 
@@ -105,35 +106,21 @@ see the L<Exporter::Tiny> documentation for details.
 =cut
 
 my $blank_re          = qr/[ \t]/;
-my $blank_or_break_re = qr/$blank_re+|$blank_re*\n$blank_re*/;
+my $blank_or_break_re = qr/$blank_re*\n?$blank_re*/;
 my $sign_re = qr/copyright(?:-holders?)?|copr\.|[©⒞Ⓒⓒ🄒🄫🅒]/i;
 my $pseudo_sign_re = qr/\(c\)/i;
 
 my $year_re = qr/\b[0-9]{4}\b/;
 my $comma_re
-	= qr/$blank_re*,$blank_or_break_re|$blank_or_break_re,$blank_re*|$blank_re*,$blank_re*|$blank_or_break_re|$blank_or_break_re/;
+	= qr/$blank_re*,$blank_or_break_re|$blank_or_break_re,?$blank_re*/;
 my $dash_re
 	= qr/$blank_re*[-˗‐‑‒–—―⁃−﹣－]$blank_or_break_re*/;
+my $owner_intro_re   = qr/\bby$blank_or_break_re/;
 my $owner_prefix_re  = qr/[(*<@[{]/;
 my $owner_initial_re = qr/[^\s!\"#$%&'()*+,.\/:;<=>?@[\\\]^_`{|}~]/;
 my $boilerplate_re
 	= qr/${comma_re}All$blank_re+Rights$blank_re+Reserved[.!]?/i;
 
-# TODO: test if \K or non-backref beneficial on perl >= 5.10
-#my $comma_tidy_re = qr/$year_re\K$comma_re(?=$year_re)/;
-#my $comma_tidy_re = qr/($year_re)$comma_re(?=$year_re)/;
-my $comma_tidy_re = qr/(?<=$year_re)$comma_re(?=$year_re)/;
-my $comma_tidied  = ', ';
-
-# TODO: test if \K beneficial on perl >= 5.10
-#my $year_begin_re = qr/(?:\A|(?<!-))($year_re)\K/;
-our $y;
-my $y_save_re   = qr/(?:(?<=[^-]($year_re))|(?<=\A($year_re)))(?{$y=$^N})/;
-my $y_next_re   = qr/(??{++$y})/;
-my $y_future_re = qr/(??{if($y<=$^N){$y=$^N;'';}else{'XXXX';}})/;
-my $years_tidy_re
-	= qr/$y_save_re(?:(?:$dash_re|$comma_tidied)($y_next_re)\b|-($year_re)$y_future_re\b)+/;
-
 # this should cause *no* false positives, and stop-chars therefore
 # exclude e.g. email address building blocks; tested against the code
 # corpus at https://codesearch.debian.net/ (tricky: its RE2 engine lacks
@@ -142,16 +129,16 @@ my $years_tidy_re
 # (?i)copyright (?:(?:claim|holder|info|information|notice|owner|ownership|statement|string)s?|in|is|to)@\b[-_@]
 # (?im)copyright (?:(?:claim|holder|info|information|notice|owner|ownership|statement|string)s?|in|is|to)[^ $]
 my $chatter_re
-	= qr/copyright$blank_or_break_re(?:(?:claim|holder|info|information|law|notice|owner|ownership|statement|string)s?|and|in|is|on|to)(?:$|@\W|[^a-zA-Z0-9@_-])/im;
+	= qr/copyright$blank_or_break_re(?:(?:claim|eq|holder|info|information|law|notice|owner|ownership|statement|string)s?|and|in|is|on|to)(?:$|@\W|[^a-zA-Z0-9@_-])/im;
 
 my $signs_re
 	= qr/(?:$sign_re|(?:^|$blank_re)$pseudo_sign_re)(?:$blank_re+(?:$sign_re|$pseudo_sign_re))*/m;
-my $years_re = qr/((?:$year_re(?:(?:$dash_re|$comma_re)$year_re)*)?)/;
-my $owners_re
-	= qr/((?:\bby$blank_or_break_re)?((?:$owner_prefix_re*$owner_initial_re\S*(?:$blank_re*\S+)*)?))/;
+my $yearspan_re = qr/$year_re(?:$dash_re$year_re)?/;
+my $years_re    = qr/$yearspan_re(?:$comma_re$yearspan_re)*/;
+my $owners_re   = qr/$owner_prefix_re*$owner_initial_re\S*(?:$blank_re*\S+)*/;
 
-my $sign_years_owners_re
-	= qr/(?:$chatter_re.*|$signs_re(?::$blank_or_break_re|$comma_re)$years_re$comma_re?$owners_re|(?:\n|\z))/s;
+my $signs_and_more_re
+	= qr/(?:$chatter_re.*|$signs_re(?::$blank_or_break_re|$comma_re)($years_re?$comma_re?$owner_intro_re?$owners_re?)|(?:\n|\z))/s;
 
 sub _generate_copyright
 {
@@ -174,15 +161,11 @@ sub _generate_copyright
 		# TODO: also parse @_ - but each separately!
 		my @block;
 		my $skipped = 0;
-		while ( $copyright =~ /$sign_years_owners_re/g ) {
-			my $years        = $1;
-			my $owners_dirty = $2;
-			my $owners       = $3 || '';
-
+		while ( $copyright =~ /$signs_and_more_re/g ) {
+			my $owners = $1;
 			if ( $globals->{threshold_before} || $globals->{threshold} ) {
 				last
 					if (!@block
-					and !length $years
 					and !length $owners
 					and ++$skipped >= ( $globals->{threshold_before}
 							|| $globals->{threshold} ) );
@@ -192,20 +175,33 @@ sub _generate_copyright
 				# "after" detects end of _current_ line so is skewed by one
 				last
 					if (@block
-					and !length $years
 					and !length $owners
 					and ++$skipped >= 1
 					+ ( $globals->{threshold_after} || $globals->{threshold} )
 					);
 			}
-			next if ( !length $years and !length $owners );
+			next if ( !length $owners );
 			$skipped = 0;
 
+			my @span = $owners =~ /\G($yearspan_re)(?:$comma_re|\Z)/gm;
+			if (@span) {
+				$owners = substr( $owners, $+[0] );
+			}
+			my $years;
+
 			# normalize
-			$years =~ s/$comma_tidy_re/$comma_tidied/g;
-			$years =~ s/$years_tidy_re/-$^N/g;
-			$owners =~ s/\s{2,}/ /g;
-			$owners =~ s/$boilerplate_re//g;
+			if (@span) {
+				my $range = Number::Range->new( map {s/$dash_re/../r} @span );
+				$years = $range->range;
+				$years =~ s/,/, /g;
+				$years =~ s/\.\./-/g;
+			}
+			if ($owners) {
+				$owners =~ s/^$owner_intro_re//g;
+				$owners =~ s/\s{2,}/ /g;
+				$owners =~ s/^$owner_intro_re//g;
+				$owners =~ s/$boilerplate_re//g;
+			}
 
 # split owner into owner_id and owner
 
